# Generated by Django 4.1.7 on 2023-03-25 23:03

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('outbox_hitcount', '0009_remove_hitlocation_name_hitlocation_city_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='hitcount',
            name='site',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site'),
        ),
    ]
