# Generated by Django 4.1.7 on 2023-03-25 22:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outbox_hitcount', '0008_remove_hitcount_browser_count_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='hitlocation',
            name='name',
        ),
        migrations.AddField(
            model_name='hitlocation',
            name='city',
            field=models.CharField(default=None, max_length=50),
        ),
        migrations.AddField(
            model_name='hitlocation',
            name='country',
            field=models.CharField(default=None, max_length=50),
        ),
    ]
