"""
Provides outbox_hitcount version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update outbox_hitcount` to change this file.

from incremental import Version

__version__ = Version("outbox_hitcount", 1, 0, 10)
__all__ = ["__version__"]
