# Generated by Django 4.1.7 on 2023-03-25 00:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outbox_hitcount', '0007_alter_hitbrowser_version_alter_hitdevice_version_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='hitcount',
            name='browser_count',
        ),
        migrations.RemoveField(
            model_name='hitcount',
            name='device_count',
        ),
        migrations.RemoveField(
            model_name='hitcount',
            name='location_count',
        ),
        migrations.RemoveField(
            model_name='hitcount',
            name='os_count',
        ),
        migrations.AlterField(
            model_name='hitbrowser',
            name='version',
            field=models.CharField(max_length=20),
        ),
        migrations.AlterField(
            model_name='hitdevice',
            name='version',
            field=models.CharField(max_length=20),
        ),
        migrations.AlterField(
            model_name='hitos',
            name='version',
            field=models.CharField(max_length=20),
        ),
    ]
