# Generated by Django 4.1.7 on 2023-03-24 01:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outbox_hitcount', '0002_hitbrowser_version_hitdevice_version_hitos_version'),
    ]

    operations = [
        migrations.AlterField(
            model_name='hitcount',
            name='end_date',
            field=models.DateTimeField(),
        ),
    ]
