# Generated by Django 4.1.7 on 2023-03-22 08:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outbox_hitcount', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='hitbrowser',
            name='version',
            field=models.CharField(default=None, max_length=10),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='hitdevice',
            name='version',
            field=models.CharField(default=None, max_length=10),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='hitos',
            name='version',
            field=models.CharField(default=None, max_length=10),
            preserve_default=False,
        ),
    ]
